import unittest
from weibo_image_rescue.utils import generate_urls_of_all_qualities


class TestGenerateUrlsOfAllQualities(unittest.TestCase):

    def test_default_quality(self):
        url = "https://wx1.sinaimg.cn/x/576bb392ly1iau1x1lgn5j21uo18g7wh.jpg"
        expected_urls = [
            "https://wx1.sinaimg.cn/original/576bb392ly1iau1x1lgn5j21uo18g7wh.jpg",
            "https://wx1.sinaimg.cn/large/576bb392ly1iau1x1lgn5j21uo18g7wh.jpg",
            "https://wx1.sinaimg.cn/mw2000/576bb392ly1iau1x1lgn5j21uo18g7wh.jpg",
            "https://wx1.sinaimg.cn/mw690/576bb392ly1iau1x1lgn5j21uo18g7wh.jpg",
            "https://wx1.sinaimg.cn/thumb150/576bb392ly1iau1x1lgn5j21uo18g7wh.jpg",
            "https://wx1.sinaimg.cn/orj360/576bb392ly1iau1x1lgn5j21uo18g7wh.jpg",
        ]
        self.assertEqual(
            sorted(generate_urls_of_all_qualities(url)), sorted(expected_urls)
        )

    def test_multiple_qualities(self):
        url = "https://wx1.sinaimg.cn/large/576bb392ly1iau1x1lgn5j21uo18g7wh.jpg"
        expected_urls = [
            "https://wx1.sinaimg.cn/large/576bb392ly1iau1x1lgn5j21uo18g7wh.jpg",
            "https://wx1.sinaimg.cn/mw690/576bb392ly1iau1x1lgn5j21uo18g7wh.jpg",
        ]
        self.assertEqual(
            sorted(generate_urls_of_all_qualities(url, ["l", "m"])),
            sorted(expected_urls),
        )

    def test_custom_and_mapped_qualities(self):
        url = "https://wx1.sinaimg.cn/abc/576bb392ly1iau1x1lgn5j21uo18g7wh.jpg"
        expected_urls = [
            "https://wx1.sinaimg.cn/x/576bb392ly1iau1x1lgn5j21uo18g7wh.jpg",
            "https://wx1.sinaimg.cn/large/576bb392ly1iau1x1lgn5j21uo18g7wh.jpg",
            "https://wx1.sinaimg.cn/asd/576bb392ly1iau1x1lgn5j21uo18g7wh.jpg",
            "https://wx1.sinaimg.cn/mw690/576bb392ly1iau1x1lgn5j21uo18g7wh.jpg",
        ]
        self.assertEqual(
            sorted(generate_urls_of_all_qualities(url, ["l", "x", "asd", "m"])),
            sorted(expected_urls),
        )


if __name__ == "__main__":
    unittest.main()



import mimetypes
import os
import time
from .constant import quality_map

def all_qualities():
    return [k for k in quality_map.keys()]

def all_qualities_str():
    return ",".join(all_qualities())


def generate_urls_of_all_qualities(url, qualities=None):
    # 如果没有提供qualities，则默认使用全部支持的 quality
    if qualities is None:
        qualities = all_qualities()
    # 映射质量值，如果不存在于quality_map中则直接使用输入值
    mapped_qualities = [quality_map.get(q, q) for q in qualities]
    # 获取原始质量部分
    parts = url.split('/')

    # 构建基础URL
    base_url = "/".join(parts[:-2]) + "/{q}/" + parts[-1]
    
    # 生成包含所有质量的URL
    urls = [base_url.replace("{q}", quality) for quality in mapped_qualities]
    
    return urls


def save_file(path, content):
    with open(path, "wb") as f:
        f.write(content)


def generate_file_name(path, url, content_type, filename):
    if filename and filename not in [".", "/"]:
        return filename

    generated = os.path.basename(path) or str(int(time.time()))
    _, existing_ext = os.path.splitext(generated)
    if existing_ext:
        return generated

    mime_type = content_type or mimetypes.guess_type(url)[0] or "application/octet-stream"
    ext = mimetypes.guess_extension(mime_type) or ".bin"
    return generated + ext



import json
import os
from .constant import cdn_file, domains
from .globalping.execute import get_cdn


def update_cdn_info():
    old_cdn_ips = []
    if os.path.exists(cdn_file) and os.path.getsize(cdn_file) > 0:
        try:
            with open(cdn_file, "r") as f:
                old_cdn_ips = json.load(f)
        except Exception:
            old_cdn_ips = []

    cdn_ips = get_cdn(domains, force=True)
    if not cdn_ips and old_cdn_ips:
        print(
            f"\nCDN refresh returned no IPs, keep existing cache. Total CDN IPs: {len(old_cdn_ips)}"
        )
        return old_cdn_ips

    with open(cdn_file, "w") as f:
        json.dump(cdn_ips, f)
    print(f"\nCDN information updated. Total CDN IPs: {len(cdn_ips)}")
    return cdn_ips


def load_cdn_info():
    if not os.path.exists(cdn_file) or os.path.getsize(cdn_file) == 0:
        return update_cdn_info()
    with open(cdn_file, "r") as f:
        cdn_ips = json.load(f)
    return cdn_ips



from urllib.parse import urlsplit, urlunsplit

import httpx

from .model import Result


def _build_direct_ip_url(url: str, ip: str) -> tuple[str, str]:
    parsed = urlsplit(url)
    if not parsed.scheme or not parsed.netloc or not parsed.hostname:
        raise ValueError(f"Invalid url: {url}")

    host = parsed.hostname
    ip_host = f"[{ip}]" if ":" in ip and not ip.startswith("[") else ip
    direct_netloc = f"{ip_host}:{parsed.port}" if parsed.port else ip_host
    direct_url = urlunsplit(
        (parsed.scheme, direct_netloc, parsed.path, parsed.query, parsed.fragment)
    )
    return direct_url, host


async def request(url, headers, ip=None, timeout=10.0):
    transport = httpx.AsyncHTTPTransport(retries=0)
    req_headers = dict(headers or {})

    if ip is None:
        async with httpx.AsyncClient(
            http2=True,
            headers=req_headers,
            transport=transport,
            timeout=timeout,
        ) as client:
            try:
                response = await client.get(url)
                return Result(
                    status=response.status_code,
                    headers=response.headers,
                    ip="dns",
                    url=url,
                    body=response.content,
                )
            except Exception as e:
                return Result(err=e, ip="dns", url=url)

    try:
        direct_url, host = _build_direct_ip_url(url, ip)
    except Exception as e:
        return Result(err=e, ip=ip, url=url)

    req_headers["Host"] = host
    async with httpx.AsyncClient(
        http2=True,
        headers=req_headers,
        transport=transport,
        timeout=timeout,
        verify=False,
    ) as client:
        try:
            response = await client.get(direct_url)
            return Result(
                status=response.status_code,
                headers=response.headers,
                ip=ip,
                url=url,
                body=response.content,
            )
        except Exception as e:
            return Result(err=e, ip=ip, url=url)



import asyncio
import unittest
from unittest.mock import AsyncMock, Mock, patch

from weibo_image_rescue import custom_transport


class TestBuildDirectIpUrl(unittest.TestCase):
    def test_build_direct_ip_url_with_ipv6_and_port(self):
        direct_url, host = custom_transport._build_direct_ip_url(
            "https://wx1.sinaimg.cn:443/mw690/a.jpg?x=1",
            "2408:4004:1000::101f",
        )
        self.assertEqual(
            direct_url,
            "https://[2408:4004:1000::101f]:443/mw690/a.jpg?x=1",
        )
        self.assertEqual(host, "wx1.sinaimg.cn")

    def test_build_direct_ip_url_invalid_url(self):
        with self.assertRaises(ValueError):
            custom_transport._build_direct_ip_url("not-a-url", "1.2.3.4")


class TestRequest(unittest.TestCase):
    def test_request_dns_success(self):
        response = Mock(status_code=200, headers={"content-type": "image/jpeg"}, content=b"x")
        client = AsyncMock()
        client.get = AsyncMock(return_value=response)
        client_cm = AsyncMock()
        client_cm.__aenter__.return_value = client

        with patch(
            "weibo_image_rescue.custom_transport.httpx.AsyncHTTPTransport"
        ) as transport_cls, patch(
            "weibo_image_rescue.custom_transport.httpx.AsyncClient",
            return_value=client_cm,
        ) as async_client_cls:
            result = asyncio.run(
                custom_transport.request(
                    "https://wx1.sinaimg.cn/mw690/a.jpg",
                    {"Accept": "image/*"},
                    ip=None,
                    timeout=3.0,
                )
            )

        self.assertEqual(result.status, 200)
        self.assertEqual(result.ip, "dns")
        self.assertEqual(result.url, "https://wx1.sinaimg.cn/mw690/a.jpg")
        self.assertEqual(result.body, b"x")
        client.get.assert_awaited_once_with("https://wx1.sinaimg.cn/mw690/a.jpg")
        kwargs = async_client_cls.call_args.kwargs
        self.assertEqual(kwargs["headers"], {"Accept": "image/*"})
        self.assertEqual(kwargs["transport"], transport_cls.return_value)
        self.assertEqual(kwargs["timeout"], 3.0)

    def test_request_dns_exception(self):
        client = AsyncMock()
        client.get = AsyncMock(side_effect=RuntimeError("boom"))
        client_cm = AsyncMock()
        client_cm.__aenter__.return_value = client

        with patch(
            "weibo_image_rescue.custom_transport.httpx.AsyncClient",
            return_value=client_cm,
        ):
            result = asyncio.run(
                custom_transport.request(
                    "https://wx1.sinaimg.cn/mw690/a.jpg",
                    {"Accept": "image/*"},
                    ip=None,
                )
            )

        self.assertIsNotNone(result.err)
        self.assertEqual(result.ip, "dns")

    def test_request_direct_ip_success_sets_host_header(self):
        response = Mock(status_code=200, headers={"content-type": "image/jpeg"}, content=b"ok")
        client = AsyncMock()
        client.get = AsyncMock(return_value=response)
        client_cm = AsyncMock()
        client_cm.__aenter__.return_value = client
        headers = {"Accept": "image/*"}

        with patch(
            "weibo_image_rescue.custom_transport.httpx.AsyncHTTPTransport"
        ), patch(
            "weibo_image_rescue.custom_transport.httpx.AsyncClient",
            return_value=client_cm,
        ) as async_client_cls:
            result = asyncio.run(
                custom_transport.request(
                    "https://wx1.sinaimg.cn/mw690/a.jpg",
                    headers,
                    ip="1.2.3.4",
                    timeout=2.5,
                )
            )

        self.assertEqual(result.status, 200)
        self.assertEqual(result.ip, "1.2.3.4")
        client.get.assert_awaited_once_with("https://1.2.3.4/mw690/a.jpg")
        called_headers = async_client_cls.call_args.kwargs["headers"]
        self.assertEqual(called_headers["Host"], "wx1.sinaimg.cn")
        self.assertEqual(headers, {"Accept": "image/*"})
        self.assertEqual(async_client_cls.call_args.kwargs["verify"], False)

    def test_request_direct_ip_invalid_url_returns_error(self):
        result = asyncio.run(
            custom_transport.request(
                "not-a-url",
                {"Accept": "image/*"},
                ip="1.2.3.4",
            )
        )
        self.assertIsNotNone(result.err)
        self.assertEqual(result.ip, "1.2.3.4")
        self.assertEqual(result.url, "not-a-url")


if __name__ == "__main__":
    unittest.main()



class MeasurementRequest:
    def __init__(self, type, target, locations, ping_options=None, http_options=None):
        self.type = type
        self.target = target
        self.locations = locations
        self.ping_options = ping_options
        self.http_options = http_options

    def to_dict(self):
        if not self.target:
            raise ValueError(".target is empty")
        if not self.locations:
            raise ValueError(".locations is empty")

        data = {
            "type": self.type,
            "target": self.target,
            "locations": [loc.to_dict() for loc in self.locations],
        }

        if self.type == "ping":
            if self.ping_options is None:
                self.ping_options = PingOptions()
            if self.ping_options.packets_count == 0:
                self.ping_options.packets_count = 1
            data["measurementOptions"] = self.ping_options.to_dict()
        elif self.type == "http":
            if self.http_options is None:
                self.http_options = HttpOptions()
            data["measurementOptions"] = self.http_options.to_dict()
        else:
            raise ValueError(f"Unknown .type: {self.type}")

        return data


class MeasurementType:
    PING = "ping"
    HTTP = "http"


class PingOptions:
    def __init__(self, packets_count=1):
        self.packets_count = packets_count

    def to_dict(self):
        return {"packets": self.packets_count}


class HttpOptions:
    def __init__(
        self,
        protocol="HTTP",
        method="HEAD",
        headers=None,
        host="",
        path="",
        query="",
        port=80,
    ):
        self.protocol = protocol
        self.method = method
        self.headers = headers or {}
        self.host = host
        self.path = path
        self.query = query
        self.port = port

    def to_dict(self):
        return {
            "protocol": self.protocol,
            "request": {
                "method": self.method,
                "headers": self.headers,
                "host": self.host,
                "path": self.path,
                "query": self.query,
            },
            "port": self.port,
        }


class Location:
    def __init__(self, region, country="", city="", limit=5):
        self.region = region
        self.country = country
        self.city = city
        self.limit = limit

    def to_dict(self):
        data = {"region": self.region, "limit": self.limit}
        if self.country:
            data["country"] = self.country
        if self.city:
            data["city"] = self.city
        return data


class ResponseOnSuccess:
    def __init__(self, id, status, results, probes_count):
        self.id = id
        self.status = status
        self.results = results
        self.probes_count = probes_count


class ResponseOnError:
    def __init__(self, error):
        self.error = error


class MeasurementResult:
    def __init__(self, result, probe):
        self.result = result
        self.probe = probe


class Probe:
    def __init__(self, location):
        self.location = location



import requests
import time
from threading import Lock
from .models import (
    MeasurementRequest,
    Location,
    Probe,
)
import socket
import ipaddress
from .config import (
    BASE_URL,
    REQUEST_TIMEOUT,
    GET_MEASUREMENT_INTERVAL,
    GET_MEASUREMENT_OVERALL_TIMEOUT,
    BASE_REQ_HEADERS,
    DEFAULT_REGIONS,
)


class Client:
    def __init__(self):
        self.session = requests.Session()
        self.etags = {}
        self.lock = Lock()

    def create_measurement(self, hostname, regions):
        if not hostname:
            raise ValueError("No hostname specified")
        if not regions:
            raise ValueError("No regions specified")

        # Globalping currently requires the sum of limits to be <= 50.
        limit_per_region = max(1, 50 // len(regions))
        max_regions = min(len(regions), 50)
        locations = [
            Location(region=r, limit=limit_per_region) for r in regions[:max_regions]
        ]
        req_body = MeasurementRequest(
            type="ping", target=hostname, locations=locations
        ).to_dict()
        url = f"{BASE_URL}/measurements"
        response = self.request("POST", url, json=req_body)
        if not response:
            raise Exception("No response from server")

        r = response.json()
        probes_count = r.get("probes_count", r.get("probesCount"))
        if probes_count == 0:
            raise Exception("No probes available")
        if not r.get("id"):
            raise Exception(f"Invalid response: {response.text}")

        return r["id"]

    def get_measurement(self, measurement_id):
        if not measurement_id:
            raise ValueError("No measurement ID specified")

        url = f"{BASE_URL}/measurements/{measurement_id}"

        with self.lock:
            self.etags.pop(url, None)

        start_time = time.time()

        while time.time() - start_time < GET_MEASUREMENT_OVERALL_TIMEOUT:
            time.sleep(GET_MEASUREMENT_INTERVAL)
            response = self.request("GET", url)

            if not response:
                # print(f"Measurement {measurement_id} in progress...", file=sys.stderr)
                continue

            r = response.json()
            if not r.get("id"):
                raise Exception(f"Invalid response: {response.text}")

            if r.get("status") == "in-progress":
                # print(f"Measurement {r['id']} in progress...", file=sys.stderr)
                continue
            elif r.get("status") == "finished":
                return r.get("results", [])

        raise TimeoutError(f"Measurement {measurement_id} timed out")

    def request(self, method, url, **kwargs):
        headers = kwargs.pop("headers", BASE_REQ_HEADERS)
        attempts = 4
        last_error = None

        for attempt in range(attempts):
            try:
                response = self.session.request(
                    method, url, headers=headers, timeout=REQUEST_TIMEOUT, **kwargs
                )
            except requests.RequestException as e:
                last_error = e
                if attempt < attempts - 1:
                    time.sleep(1.5 * (2**attempt))
                    continue
                raise

            if response.status_code in (429, 503) and attempt < attempts - 1:
                retry_after = response.headers.get("Retry-After")
                try:
                    delay = float(retry_after) if retry_after else 1.5 * (2**attempt)
                except ValueError:
                    delay = 1.5 * (2**attempt)
                time.sleep(delay)
                continue

            if response.status_code == 304:
                return None
            response.raise_for_status()
            return response

        if last_error:
            raise last_error
        raise Exception(f"Request failed after retries: {method} {url}")

    def resolve(self, hostname, locations=None):
        if locations is None:
            locations = DEFAULT_REGIONS

        mID = self.create_measurement(hostname, locations)
        mResults = self.get_measurement(mID)

        IPs = []
        for result in mResults:
            resolved = result.get("result", {}).get("resolvedAddress")
            if not resolved:
                continue
            try:
                ipaddress.ip_address(resolved)
                IPs.append(resolved)
            except ValueError:
                IPs.append(socket.gethostbyname(resolved))
        return IPs

    def probes(self):
        probes = self.get_probes()

        locations = [
            probe["location"]["region"]
            for probe in probes
            if probe["location"]["region"]
        ]
        return locations

    def locations(self):
        return DEFAULT_REGIONS

    def get_probes(self):
        url = f"{BASE_URL}/probes"
        response = self.request("GET", url)
        probes = response.json()
        return [Probe(**probe) for probe in probes]



from concurrent.futures import ThreadPoolExecutor, as_completed
import queue
import socket
from tqdm import tqdm
from .client import Client


def get_cdn(hostnames, force=False):
    provider = Client()
    
    try:
        locations = provider.locations()
    except Exception as e:
        raise Exception(f"failed to get locations: {str(e)}")

    locations = unique(locations)
    print(f"Using {len(locations)} locations.")

    ch = queue.Queue(len(hostnames))
    progress_bar = tqdm(total=len(hostnames), desc="Resolving hostnames")

    def resolve_hostname(hostname):
        try:
            ips = provider.resolve(hostname, locations)
            ch.put(ips)
        except Exception as e:
            # print(f'Failed to resolve "{hostname}": {str(e)}', file=sys.stderr)
            ch.put(None)
        finally:
            progress_bar.update(1)

    with ThreadPoolExecutor(max_workers=min(len(hostnames), 2)) as executor:
        futures = [executor.submit(resolve_hostname, h) for h in hostnames]
        for future in as_completed(futures):
            pass

    progress_bar.close()

    resolves = []
    if not force:
        resolves = resolves

    while not ch.empty():
        result = ch.get()
        if result is not None:
            resolves.extend(result)

    if not resolves:
        # Fallback to local DNS when Globalping is unavailable/rate-limited.
        for hostname in hostnames:
            try:
                _, _, ips = socket.gethostbyname_ex(hostname)
                resolves.extend(ips)
            except Exception:
                pass

    ip_list = unique_ips(resolves)
    return ip_list

def unique(items):
    return list(set(items))


def unique_ips(ip_list):
    seen = set()
    unique_list = []
    for item in ip_list:
        if item not in seen:
            unique_list.append(item)
            seen.add(item)
    return unique_list



import argparse
from .utils import all_qualities_str
from .constant import output_dir

def parse_args(args):
    parser = argparse.ArgumentParser(description="Weibo Image Rescue Tool")
    parser.add_argument(
        "-u", "--update-cdn", action="store_true", help="Update CDN information only"
    )
    parser.add_argument(
        "-f", type=str, help="Force update CDN information and rescue image from URL"
    )
    parser.add_argument(
        "--force-update",
        action="store_true",
        help="Force update CDN information before rescuing the positional image URL",
    )
    parser.add_argument(
        "-q",
        type=str,
        default=all_qualities_str(),
        help="Specify qualities to rescue, can be a single value or multiple comma-separated values, e.g., l,ml,m,o,t,ol (default: all qualities)",
    )
    parser.add_argument(
        "-o",
        "--output-dir",
        type=str,
        default=f"./{output_dir}/",
        help="Output directory for rescued image files",
    )
    parser.add_argument(
        "--workers",
        type=int,
        default=20,
        help="Max concurrent requests when probing CDN IPs (default: 20)",
    )
    parser.add_argument(
        "--timeout",
        type=float,
        default=10.0,
        help="Request timeout in seconds (default: 10)",
    )
    parser.add_argument("image_url", nargs="?", type=str, help="Image URL to rescue")
    parsed = parser.parse_args(args)

    if parsed.workers < 1:
        parser.error("--workers must be >= 1")
    if parsed.timeout <= 0:
        parser.error("--timeout must be > 0")
    if parsed.update_cdn and (parsed.f or parsed.image_url):
        parser.error("-u/--update-cdn cannot be used with image URL arguments")
    if parsed.update_cdn and parsed.force_update:
        parser.error("-u/--update-cdn cannot be used with --force-update")
    if parsed.f and parsed.image_url:
        parser.error("Provide URL using either positional image_url or -f, not both")
    if parsed.f and parsed.force_update:
        parser.error("-f already forces update; do not use with --force-update")
    if parsed.force_update and not parsed.image_url:
        parser.error("--force-update requires positional image_url")

    return parsed



import unittest
from types import SimpleNamespace
from unittest.mock import patch

from weibo_image_rescue import main as main_module


def _args(**overrides):
    base = {
        "update_cdn": False,
        "f": None,
        "image_url": "https://wx1.sinaimg.cn/mw690/a.jpg",
        "force_update": False,
        "q": "l,m",
        "output_dir": "./images/",
        "workers": 20,
        "timeout": 10.0,
    }
    base.update(overrides)
    return SimpleNamespace(**base)


class TestMain(unittest.TestCase):
    def test_main_update_cdn_only(self):
        args = _args(update_cdn=True)
        with patch(
            "weibo_image_rescue.main.parse_args", return_value=args
        ) as mock_parse, patch(
            "weibo_image_rescue.main.update_cdn_info"
        ) as mock_update, patch(
            "weibo_image_rescue.main.load_cdn_info"
        ) as mock_load, patch(
            "weibo_image_rescue.main.rescue"
        ) as mock_rescue:
            main_module.main(["-u"])

        mock_parse.assert_called_once_with(["-u"])
        mock_update.assert_called_once()
        mock_load.assert_not_called()
        mock_rescue.assert_not_called()

    def test_main_force_mode_uses_f_url_and_updates_cdn(self):
        args = _args(
            f="https://wx2.sinaimg.cn/mw690/f.jpg",
            image_url=None,
            q="l, m,,o",
            output_dir="/tmp/out",
            workers=7,
            timeout=3.5,
        )
        with patch("weibo_image_rescue.main.parse_args", return_value=args), patch(
            "weibo_image_rescue.main.update_cdn_info"
        ) as mock_update, patch(
            "weibo_image_rescue.main.load_cdn_info", return_value=["1.1.1.1"]
        ) as mock_load, patch(
            "weibo_image_rescue.main.generate_urls_of_all_qualities",
            return_value=["u1", "u2"],
        ) as mock_generate, patch(
            "weibo_image_rescue.main.rescue"
        ) as mock_rescue:
            main_module.main(["-f", args.f])

        mock_update.assert_called_once()
        mock_load.assert_called_once()
        mock_generate.assert_called_once_with(args.f, ["l", "m", "o"])
        mock_rescue.assert_called_once_with(
            ["u1", "u2"],
            ["1.1.1.1"],
            "/tmp/out",
            workers=7,
            timeout=3.5,
        )

    def test_main_force_update_with_positional_url(self):
        args = _args(force_update=True, f=None, image_url="https://wx3.sinaimg.cn/mw690/z.jpg")
        with patch("weibo_image_rescue.main.parse_args", return_value=args), patch(
            "weibo_image_rescue.main.update_cdn_info"
        ) as mock_update, patch(
            "weibo_image_rescue.main.load_cdn_info", return_value=[]
        ), patch(
            "weibo_image_rescue.main.generate_urls_of_all_qualities", return_value=["u"]
        ), patch("weibo_image_rescue.main.rescue"):
            main_module.main(["--force-update", args.image_url])

        mock_update.assert_called_once()

    def test_main_requires_image_url(self):
        args = _args(f=None, image_url=None, force_update=False)
        with patch("weibo_image_rescue.main.parse_args", return_value=args), patch(
            "weibo_image_rescue.main.print"
        ) as mock_print, patch(
            "weibo_image_rescue.main.load_cdn_info"
        ) as mock_load, patch(
            "weibo_image_rescue.main.rescue"
        ) as mock_rescue:
            main_module.main([])

        mock_print.assert_called_once_with("Image URL is required.")
        mock_load.assert_not_called()
        mock_rescue.assert_not_called()


if __name__ == "__main__":
    unittest.main()



import unittest

from weibo_image_rescue.args import parse_args
from weibo_image_rescue.utils import generate_file_name


class TestArgs(unittest.TestCase):
    def test_update_cdn_conflicts_with_url(self):
        with self.assertRaises(SystemExit):
            parse_args(["-u", "https://wx4.sinaimg.cn/mw690/a.jpg"])

    def test_f_conflicts_with_positional_url(self):
        with self.assertRaises(SystemExit):
            parse_args(["-f", "https://wx4.sinaimg.cn/mw690/a.jpg", "https://x/a.jpg"])

    def test_force_update_flag(self):
        args = parse_args(["--force-update", "https://wx4.sinaimg.cn/mw690/a.jpg"])
        self.assertTrue(args.force_update)
        self.assertEqual(args.image_url, "https://wx4.sinaimg.cn/mw690/a.jpg")

    def test_workers_and_timeout_validation(self):
        with self.assertRaises(SystemExit):
            parse_args(["--workers", "0", "https://wx4.sinaimg.cn/mw690/a.jpg"])
        with self.assertRaises(SystemExit):
            parse_args(["--timeout", "0", "https://wx4.sinaimg.cn/mw690/a.jpg"])

    def test_force_update_requires_positional_url(self):
        with self.assertRaises(SystemExit):
            parse_args(["--force-update"])


class TestGenerateFileName(unittest.TestCase):
    def test_keep_existing_extension(self):
        result = generate_file_name(
            "/path/to/image.jpg",
            "https://wx4.sinaimg.cn/mw690/image.jpg",
            "image/jpeg",
            None,
        )
        self.assertEqual(result, "image.jpg")

    def test_add_extension_when_missing(self):
        result = generate_file_name(
            "/path/to/image",
            "https://wx4.sinaimg.cn/mw690/image",
            "image/jpeg",
            None,
        )
        self.assertEqual(result, "image.jpg")


if __name__ == "__main__":
    unittest.main()



class Result:
    def __init__(self, err=None, headers=None, ip=None, url = None,body=None, status=None):
        self.err = err
        self.headers = headers
        self.ip = ip
        self.body = body
        self.status = status
        self.url = url



from urllib.parse import urlparse
import os
from tqdm import tqdm
import asyncio
from .args import parse_args
from .constant import base_headers
from .cdn import load_cdn_info, update_cdn_info
from .custom_transport import request
from .utils import generate_file_name, generate_urls_of_all_qualities, save_file


def save(result, output_path, filename):
    u = urlparse(result.url)
    filename = generate_file_name(
        u.path, result.url, result.headers.get("content-type"), filename
    )
    path = os.path.join(output_path, filename)
    if not os.path.exists(output_path):
        os.makedirs(output_path)
    save_file(path, result.body)
    print(f"Saved {result.url} to {path}")


async def rescue_async(
    urls, ips, output_path, filename=None, workers=20, timeout=10.0
):
    total_tasks = len(urls) * (len(ips) + 1)
    result = None
    semaphore = asyncio.Semaphore(max(1, workers))

    with tqdm(total=total_tasks, desc="Rescuing") as pbar:
        for url in urls:
            stop_event = asyncio.Event()
            ip_candidates = [None] + list(ips)

            async def run_task(ip):
                if stop_event.is_set():
                    return None
                async with semaphore:
                    if stop_event.is_set():
                        return None
                    task_result = await request(url, base_headers, ip, timeout=timeout)
                    pbar.update(1)
                    if task_result and task_result.status == 200:
                        stop_event.set()
                        return task_result
                    return None

            pending_tasks = [asyncio.create_task(run_task(ip)) for ip in ip_candidates]
            try:
                for completed in asyncio.as_completed(pending_tasks):
                    task_result = await completed
                    if task_result and task_result.status == 200:
                        result = task_result
                        break
            finally:
                for task in pending_tasks:
                    if not task.done():
                        task.cancel()
                await asyncio.gather(*pending_tasks, return_exceptions=True)

            if result is not None:
                break

        pbar.n = pbar.total
        pbar.refresh()

    if not result:
        print(f"[FAILED] Unfortunately, all {total_tasks} resolves failed.")
        return None

    print(f"[SUCCESS] {result.url} | {result.ip} | {len(result.body)}")
    save(result, output_path, filename)
    return result


def rescue(urls, ips, output_path, filename=None, workers=20, timeout=10.0):
    return asyncio.run(
        rescue_async(
            urls,
            ips,
            output_path,
            filename=filename,
            workers=workers,
            timeout=timeout,
        )
    )


def main(args=None):
    if args is None:
        args = parse_args(None)
    else:
        args = parse_args(args)

    if args.update_cdn:
        update_cdn_info()
        return

    if args.f:
        update_cdn_info()
        image_url = args.f
    else:
        image_url = args.image_url
        if args.force_update:
            update_cdn_info()

    if not image_url:
        print("Image URL is required.")
        return

    ips = load_cdn_info()

    if not ips:
        print("No cached resolves found. Continue with DNS fallback only.")

    print(f"Using {len(ips)} cached resolves.")
    qualities = [item.strip() for item in args.q.split(",") if item.strip()]
    urls = generate_urls_of_all_qualities(image_url, qualities)
    rescue(
        urls,
        ips,
        args.output_dir,
        workers=args.workers,
        timeout=args.timeout,
    )


if __name__ == "__main__":
    main()



# Constants
BASE_URL = "https://api.globalping.io/v1"
REQUEST_TIMEOUT = 15
GET_MEASUREMENT_INTERVAL = 5
GET_MEASUREMENT_OVERALL_TIMEOUT = 60

# Default headers
BASE_REQ_HEADERS = {
    "content-type": "application/json",
    "accept": "application/json",
    "accept-encoding": "br, gzip, deflate",
    "user-agent": "WeiboImageRescue/1.0 (miles/image-rescue)"
}

# Default regions
DEFAULT_REGIONS = ["Northern Africa", "Eastern Africa", "Middle Africa", "Southern Africa", "Western Africa", "Caribbean", 
                   "Central America", "South America", "Northern America", "Central Asia", "Eastern Asia", 
                   "South-eastern Asia", "Southern Asia", "Western Asia", "Eastern Europe", "Northern Europe", 
                   "Southern Europe", "Western Europe", "Australia and New Zealand", "Melanesia", "Micronesia", "Polynesia"]

# test retions
# DEFAULT_REGIONS = ["Northern Africa", "Eastern Africa","Northern America"]


import unittest
from unittest.mock import patch

from weibo_image_rescue.globalping import execute


class DummyProgressBar:
    def __init__(self, *args, **kwargs):
        pass

    def update(self, _):
        pass

    def close(self):
        pass


class TestGlobalpingExecute(unittest.TestCase):
    def test_unique(self):
        items = ["a", "b", "a"]
        result = execute.unique(items)
        self.assertEqual(set(result), {"a", "b"})

    def test_unique_ips_preserves_order(self):
        self.assertEqual(
            execute.unique_ips(["1.1.1.1", "2.2.2.2", "1.1.1.1", "3.3.3.3"]),
            ["1.1.1.1", "2.2.2.2", "3.3.3.3"],
        )

    def test_get_cdn_collects_from_provider(self):
        class Provider:
            def locations(self):
                return ["asia", "asia", "europe"]

            def resolve(self, hostname, locations):
                self.last_locations = locations
                if hostname == "wx1.sinaimg.cn":
                    return ["1.1.1.1", "2.2.2.2"]
                return ["2.2.2.2", "3.3.3.3"]

        provider = Provider()
        with patch(
            "weibo_image_rescue.globalping.execute.Client", return_value=provider
        ), patch("weibo_image_rescue.globalping.execute.tqdm", DummyProgressBar):
            result = execute.get_cdn(["wx1.sinaimg.cn", "wx2.sinaimg.cn"], force=True)

        self.assertEqual(set(result), {"1.1.1.1", "2.2.2.2", "3.3.3.3"})
        self.assertEqual(set(provider.last_locations), {"asia", "europe"})

    def test_get_cdn_fallbacks_to_local_dns_when_provider_has_no_results(self):
        class Provider:
            def locations(self):
                return ["asia"]

            def resolve(self, hostname, locations):
                raise RuntimeError("globalping unavailable")

        provider = Provider()

        def fake_gethostbyname_ex(hostname):
            if hostname == "wx1.sinaimg.cn":
                return (hostname, [], ["4.4.4.4", "5.5.5.5"])
            return (hostname, [], ["5.5.5.5", "6.6.6.6"])

        with patch(
            "weibo_image_rescue.globalping.execute.Client", return_value=provider
        ), patch(
            "weibo_image_rescue.globalping.execute.tqdm", DummyProgressBar
        ), patch(
            "weibo_image_rescue.globalping.execute.socket.gethostbyname_ex",
            side_effect=fake_gethostbyname_ex,
        ):
            result = execute.get_cdn(["wx1.sinaimg.cn", "wx2.sinaimg.cn"], force=True)

        self.assertEqual(result, ["4.4.4.4", "5.5.5.5", "6.6.6.6"])

    def test_get_cdn_wraps_location_errors(self):
        class Provider:
            def locations(self):
                raise RuntimeError("boom")

        with patch(
            "weibo_image_rescue.globalping.execute.Client", return_value=Provider()
        ):
            with self.assertRaises(Exception) as cm:
                execute.get_cdn(["wx1.sinaimg.cn"])

        self.assertIn("failed to get locations", str(cm.exception))


if __name__ == "__main__":
    unittest.main()



base_headers = {
    "Accept": "image/avif,image/webp,image/apng,image/svg+xml,image/*,*/*;q=0.8",
    "Accept-Encoding": "gzip, deflate, br",
    "Accept-Language": "zh-CN,zh;q=0.9",
    "Referer": "https://weibo.com/",
    "Sec-Ch-Ua": 'Google Chrome";v="119", "Chromium";v="119", "Not?A_Brand";v="24',
    "Sec-Ch-Ua-Mobile": "?0",
    "Sec-Ch-Ua-Platform": "Windows",
    "Sec-Fetch-Dest": "image",
    "Sec-Fetch-Mode": "no-cors",
    "Sec-Fetch-Site": "cross-site",
    # "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/119.0.0.0 Safari/537.36",
}

output_dir = "images"


cdn_file = "cdn.json"


# public_dns_servers = [
#     "8.8.8.8",        # Google Public DNS (USA)
#     "8.8.4.4",        # Google Public DNS (USA)
#     "1.1.1.1",        # Cloudflare DNS (Global)
#     "1.0.0.1",        # Cloudflare DNS (Global)
#     "9.9.9.9",        # Quad9 DNS (Global)
#     "149.112.112.112",# Quad9 Secondary DNS (Global)
#     "208.67.222.222", # OpenDNS (Global)
#     "208.67.220.220", # OpenDNS Secondary (Global)
#     "180.76.76.76",   # Baidu Public DNS (China)
#     "114.114.114.114",# 114 DNS (China)
#     "168.95.1.1",     # HiNet DNS (Taiwan)
#     "77.88.8.8",      # Yandex DNS (Russia)
#     "77.88.8.1",      # Yandex DNS (Russia)
#     "74.82.42.42",    # Hurricane Electric DNS (USA)
#     "129.250.35.250", # NTT DNS (Japan)
#     "4.2.2.1",        # Level3 DNS (USA)
#     "4.2.2.2",        # Level3 DNS (USA)
#     "8.26.56.26",     # Comodo Secure DNS (Global)
#     "8.20.247.20",    # Comodo Secure DNS (Global)
#     "64.6.64.6",      # Verisign DNS (Global)
#     "64.6.65.6",      # Verisign DNS (Global)
#     "1.2.4.8",        # DNSPod Public DNS+ (China)
# ]


domains = ["wx1.sinaimg.cn", "wx2.sinaimg.cn", "wx3.sinaimg.cn", "wx4.sinaimg.cn"]


quality_map = {
    "ol": "original",
    "l": "large",
    "ml": "mw2000",
    "m": "mw690",
    "t": "thumb150",
    "o": "orj360",
}


VALID_IMAGE_URL = "https://wx1.sinaimg.cn/mw690/576bb392ly1iau1x1lgn5j21uo18g7wh.jpg"
INVALID_IMAGE_URL = "https://wx3.sinaimg.cn/mw690/41d692acgy1hrnmhkdmqfj20be0beaag.jpg"
ABSOLUTE_INVALID_IMAGE_URL = (
    "https://wx3.sinaimg.cn/mw2000/ede975b2gy1hrosk73sx0j20ib0i4q9c.jpg"
)



import json
import os
import tempfile
import unittest
from unittest.mock import patch

from weibo_image_rescue import cdn


class TestCdn(unittest.TestCase):
    def test_update_cdn_info_keeps_old_cache_when_refresh_empty(self):
        with tempfile.TemporaryDirectory() as tmpdir:
            cdn_path = os.path.join(tmpdir, "cdn.json")
            old_ips = ["1.1.1.1", "2.2.2.2"]
            with open(cdn_path, "w", encoding="utf-8") as f:
                json.dump(old_ips, f)

            with patch("weibo_image_rescue.cdn.cdn_file", cdn_path), patch(
                "weibo_image_rescue.cdn.get_cdn", return_value=[]
            ):
                result = cdn.update_cdn_info()

            self.assertEqual(result, old_ips)
            with open(cdn_path, "r", encoding="utf-8") as f:
                self.assertEqual(json.load(f), old_ips)

    def test_update_cdn_info_overwrites_cache_when_refresh_succeeds(self):
        with tempfile.TemporaryDirectory() as tmpdir:
            cdn_path = os.path.join(tmpdir, "cdn.json")
            with open(cdn_path, "w", encoding="utf-8") as f:
                f.write("{invalid json")

            new_ips = ["3.3.3.3"]
            with patch("weibo_image_rescue.cdn.cdn_file", cdn_path), patch(
                "weibo_image_rescue.cdn.get_cdn", return_value=new_ips
            ):
                result = cdn.update_cdn_info()

            self.assertEqual(result, new_ips)
            with open(cdn_path, "r", encoding="utf-8") as f:
                self.assertEqual(json.load(f), new_ips)

    def test_load_cdn_info_calls_update_when_file_missing(self):
        with tempfile.TemporaryDirectory() as tmpdir:
            cdn_path = os.path.join(tmpdir, "missing.json")
            with patch("weibo_image_rescue.cdn.cdn_file", cdn_path), patch(
                "weibo_image_rescue.cdn.update_cdn_info", return_value=["9.9.9.9"]
            ) as mock_update:
                result = cdn.load_cdn_info()

            self.assertEqual(result, ["9.9.9.9"])
            mock_update.assert_called_once()

    def test_load_cdn_info_reads_existing_cache(self):
        with tempfile.TemporaryDirectory() as tmpdir:
            cdn_path = os.path.join(tmpdir, "cdn.json")
            ips = ["8.8.8.8", "8.8.4.4"]
            with open(cdn_path, "w", encoding="utf-8") as f:
                json.dump(ips, f)

            with patch("weibo_image_rescue.cdn.cdn_file", cdn_path):
                result = cdn.load_cdn_info()

            self.assertEqual(result, ips)


if __name__ == "__main__":
    unittest.main()
